package ben.namespace;
import java.util.ArrayList;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.view.View;

public class DrawingOverlay extends View
{
    //paint defines stroke, color, etc.
	private Paint paint = new Paint();
    
	//lists of geometric entities to draw on overlay
    private ArrayList<Line> lines = new ArrayList<Line>();
    private ArrayList<Point> points = new ArrayList<Point>();
    private ArrayList<Point> circles = new ArrayList<Point>();
    
    public DrawingOverlay(Context context) 
    {
        super(context);	//calls view constructor
        
        //set paint settings
        paint.setStrokeWidth(5);
        
        //background must be transparent to see image behind it
        setBackgroundColor(Color.TRANSPARENT);
    }
    
    //manage lists of shapes
    public void setLines(ArrayList<Line> lines)
    {
    	this.lines = lines;
    }
    public void setCircles(ArrayList<Point> circles)
    {
    	this.circles = circles;
    }
    public void setPoints(ArrayList<Point> points)
    {
    	this.points = points;
    }
    
    @Override
    public void onDraw(Canvas canvas) 
    /*called by container by invalidate()*/
    {
    	//paint each line in blue
    	paint.setColor(Color.BLUE);
    	for (Line line : lines)
    		canvas.drawLine(line.x1, line.y1, line.x2, line.y2, paint);
    	
    	//paint each circle in cyan
    	paint.setColor(Color.CYAN);
    	for(Point point: circles)
    		canvas.drawCircle(point.x,point.y, 20, paint);
    	
    	//paint each point in red
    	paint.setColor(Color.RED);
    	for(Point point: points)
    		canvas.drawCircle(point.x, point.y, 5, paint);
    } 
}
