package ben.namespace;

import java.text.DecimalFormat;
import java.util.ArrayList;

public class Logic 
{
	HelloWorldActivity graphics;
	
	private Line firstLine;
	private Line secondLine;
	private ArrayList<Point> corners = new ArrayList<Point>();
	
	private Step step = Step.NONE;
	private boolean usePoints = false;
	private boolean adjusting = false;
	
	public Logic(HelloWorldActivity graphics)
	{
		this.graphics = graphics;
		clearAngle();
	}
	
    //calculation
    public void calcAngles()
    {
    	/*
    	 * lower case letters refer to sides, upper to angles
    	 * a side is opposite to the angle with the same letter
    	 *calculate angle using law of cosines c^2 = a^2 + b^2 -2abCOS(C)
    	 *translates to C = ARC-COS( (a^2 + b^2 - c^2) / (2ab) )
    	 */
    	
    	//find sides lengths of a, b, and c of triangle
    	double aLength = firstLine.length();
    	double bLength = secondLine.length();
    	double cLength = Point.distance(secondLine.getPoint2(),firstLine.getPoint2());
    	
    	//find angle C, the angle between sides a and b
    	double numerator = Math.pow(aLength,2) + Math.pow(bLength,2) - Math.pow(cLength,2);
    	double denominator = 2 * aLength * bLength;
    	double angle = Math.toDegrees(Math.acos(numerator / denominator));	//Math.acos() returns radians, must convert
    	
    	//put angle in two digit format
    	DecimalFormat twoDForm = new DecimalFormat("#.##");
        angle = Double.valueOf(twoDForm.format(angle));
    	
        //display angle in status viewer
        graphics.sendStatus(graphics.getStringResource(R.string.message_angle1) +
        					" " + angle + " " + 
        					graphics.getStringResource(R.string.message_angle2));
    }
	
    //drawing
    private void drawCircles()
    {
    	ArrayList<Point> circles = new ArrayList<Point>();
    	
    	circles.add(firstLine.getPoint1());
    	if(firstLine.getPoint2().x != 0)
    		circles.add(firstLine.getPoint2());
    	if(secondLine.getPoint2().x != 0)
    	circles.add(secondLine.getPoint2());
    	
    	graphics.drawCircles(circles);
    }
    private void drawLines()
    {
    	ArrayList<Line> lines = new ArrayList<Line>();
    	
    	if(firstLine.getPoint2().x != 0)
    		lines.add(firstLine);
    	if(secondLine.getPoint2().x != 0)
    		lines.add(secondLine);
    	
    	graphics.drawLines(lines);
    }
    
    //drawing settings
  	public void resetDrawingSettings()
  	{
  		step = Step.NONE;
  		usePoints = false;
  		adjusting = false;
  	}
    public void setupAngleDrawing(boolean usePoints)
    {
    	clearAngle();
    	adjusting = false;
    	this.usePoints = usePoints;
    	step = Step.VERTEX;
    	graphics.sendStepStatus(step);
    }
    
    //angle adjusting
    public void setupAngleAdjusting()
    {
    	adjusting = true;
    }
    public void endAngleAdjusting()
    {
    	adjusting = false;
    }
    
    //click processing 
    public void clicked(int xClick, int yClick)
    /* Accepts a click. Draws an angle using depending on settings*/
    {
    	if(!adjusting)
    	{
    		//get point according to settings
	    	Point loc = getPoint(xClick, yClick);
	    	int xLoc = loc.x;
	    	int yLoc = loc.y;
	    	
	    	switch(step)	//define different parts of angle depending on current logic.getStep()
	    	{
	    		case VERTEX:	//define vertex of angle
	    		{
	    			setVertex(xLoc, yLoc);
	    			step = Step.FIRST;
	    			break;
	    		}
	    		case FIRST:		//define first line of angle
	    		{
	    			setFirst(xLoc, yLoc);
	    			step = Step.SECOND;
	    			break;
	    		}
	    		case SECOND:	//define second line of angle
	    		{
	    			setSecond(xLoc, yLoc);
	    			step = Step.DONE;
	    			break;
	    		}
	    	}
	    	
	    	drawCircles();
	    	drawLines();
	    	graphics.sendStepStatus(step);	//update status message
    	}
    }
    public void moved(int xClick, int yClick)
    {
    	//get point to move to according to settings
    	Point loc = getPoint(xClick, yClick);
    	
    	if(adjusting)
    	{
    		double distanceVertex = Point.distance(firstLine.getPoint1(),loc);
    		double distanceFirst = Point.distance(firstLine.getPoint2(),loc);
    		double distanceSecond = Point.distance(secondLine.getPoint2(), loc);
    		
    		if(distanceVertex < distanceFirst && distanceVertex < distanceSecond)
    			setVertex(loc.x,loc.y);
    		else if(distanceFirst < distanceSecond)
    			setFirst(loc.x,loc.y);
    		else
    			setSecond(loc.x,loc.y);
    	}
    	
    	drawCircles(); //update overlay
    	drawLines();
    }
    private Point getPoint(int xClick, int yClick)
    {
    	int xLoc = 0;
    	int yLoc = 0;
    	
    	if(usePoints == false)	//in manual mode, use exact location clicked
    	{
    		xLoc = xClick;
    		yLoc = yClick;
    	}
    	else	//in automatic mode, use location of point closest to location clicked
    	{
    		Point max = findNearestCorner(xClick,yClick);
    		xLoc = max.x;
    		yLoc = max.y;
    	}
    	
    	return new Point(xLoc,yLoc,0);
    }

    //set up lines
    private void setVertex(int xLoc, int yLoc)
    {
    	firstLine.x1 = xLoc;
    	firstLine.y1 = yLoc;
		secondLine.x1 = xLoc;
		secondLine.y1 = yLoc;
    }
    private void setFirst(int xLoc, int yLoc)
    {
    	firstLine.x2 = xLoc;
    	firstLine.y2 = yLoc;
    }
    private void setSecond(int xLoc, int yLoc)
    {
    	secondLine.x2 = xLoc;
		secondLine.y2 = yLoc;
    }
    
    //corners
    public void findCorners()
    /* calculate corners of image, display on overlay*/
    {
    	//find corners
    	corners = CornerFinder.findCorners(graphics.getBitmap());
    	
    	//remove corners under buttons
    	ArrayList<Point> remove = new ArrayList<Point>();
    	for (Point point : corners) 
    	{
    		if(point.y < graphics.getButtonHeight())
    			remove.add(point);
		}
    	corners.removeAll(remove);
    	
    	//add corners to overlay
    	graphics.drawPoints(corners);
    }
    private Point findNearestCorner(int xLoc, int yLoc)
    //finds nearest corner to location (xLoc,yLoc)
    {
    	Point click = new Point(xLoc,yLoc,0);
    	Point closest = corners.get(1);
    	
		for (Point point : corners) 
			if(Point.distance(click, point) < Point.distance(click,closest))	//if closer than closest
				closest = point;
		
		return closest;
    }
	
	//angle management
    public void clearAngle()
    //clear angle lines from overlay, reset lines
    {
    	//update data
    	firstLine = new Line(0,0,0,0);
    	secondLine = new Line(0,0,0,0);
    	
    	drawCircles();
    	drawLines();
    }
	
    //corner management
    public boolean areCornersPresent()
    {
    	if(corners != null)
    		if(corners.size() > 0)
    			return true;
    		
    	return false;
    }
	public void resetCorners()
	{
		this.corners = new ArrayList<Point>();
		graphics.drawPoints(corners);
	}
}